'use strict'

const { program } = require('commander');
const fs = require('fs')
const diffParse = require('parse-diff')
var libxmljs = require("libxmljs");
const xml = require('./xml');

const app = {
  run() {
    program
      .version('0.0.4')
      .description('Compare cobertura.xml generated by jest with a diff patch file')
      .option('-c, --cobertura <cobertura>', 'The file generated by the (feature) branch you want to check', 'coverage/cobertura.xml')
      .option('-d, --diff <diff>', 'The diff patch file ', 'diff.patch')
      .option('-p, --project <project>', 'The project path form the workspace root', '')
      .option('-t, --threshold <threshold>','The minimal threshold code should have', '0.70')
      .option('-v, --variance <variance>','Allow for variance, e.g. allow 0.05 to allow a 5% decrease', '0');
    program.parse(process.argv);
    const options = program.opts();

    console.info(`##Cobertura Diff Coverage ## Version: ${options['version']}`);

    this.validateOptions(options);

    // this example reads the file synchronously
    // you can read it asynchronously also
    let xmlString = fs.readFileSync(options.cobertura, "utf8");
    let diffstring = fs.readFileSync(options.diff, "utf8");

    var diffFile = diffParse(diffstring);

    let exitCode = 0;
    const minCov = parseFloat(options.new*100);

    if (options.variance!='0') {
      console.log(`Allowed variance is ${options.variance}`);
    }
    
    var xmlDoc = xml.parseXML(xmlString);

    var packages = [];
    if(Array.isArray(xmlDoc.coverage.packages))
    {
      packages = xmlDoc.coverage.packages
    }
    else{
      packages.push(xmlDoc.coverage.packages)
    }
    var linesToCover = 0
    var coveredLines = 0
    packages.forEach((key, index) => {
      try {
        let patchObject = diffFile.find(patch => patch.to === options.project+key.class.filename)
        if(patchObject.chunks)
        {
          //console.log(patchObject.chunks)
          
          //console.log(key.class.lines.array);

          //coletar o chunk e verifica se existe interseccao nas linhas do coberturaxml
          patchObject.chunks.forEach(element => {
            var chunck = key.class.lines.array.find(line => {
              console.log
              return line.line.number >= element.newStart && line.line.number <= element.newStart+element.newLines
            })
            if(chunck)
            {
              console.log("Found a new match on: "+element.newStart)
            }
          });
          //console.log(key.class.lines.array.map(this.convertLine))
          //expandir as changes e verificar exatamente qual linha foi moficiada
          //bater as linhas modificadas com a cobertura e armazenar a informacao do cobertura
          //sumarizar as linhas que deveriam estar cobertas e que de fato foram cobertas e fechar o processo com isso
        }
          
        
      } catch(err) {
        console.error(err)
        exitCode = 1;
      }
    })
    
    console.info(`Coverage check is completed.`);
    if(exitCode > 0)
    {
      console.info(`Something went wrong, check the logs for more info.`);
    } else {
      console.info(`All the checks have passed succesfully.`);
    }
    process.exit(exitCode);
  },
  validateOptions: function(options)
  {
    if(!options.project)
    {
      console.log("The project path is mandatory, no project path option passed on -p");
      process.exit(1);
    }
    
  },
  verifyIntersection: function(chunk){
    
  }
}

module.exports = app;
